@@ -14,6 +14,8 @@ from datamate.directory import (
     get_check_size_on_init,
     set_verbosity_level,
     root,
+    set_root_context,
+    reset_scope,
 )
 from datamate.namespaces import Namespace, namespacify
 
